@@ -12,6 +12,7 @@ from pandas.core.dtypes.api import is_list_like
 import pandas.util.testing as tm
 from pandas.util.testing import (ensure_clean,
                                  assert_is_valid_plot_return_object)
+import pandas.util._test_decorators as td
 
 import numpy as np
 from numpy import random
@@ -23,8 +24,6 @@ from pandas.plotting._tools import _flatten
 This is a common base class used for various plotting tests
 """
 
-tm._skip_module_if_no_mpl()
-
 
 def _skip_if_no_scipy_gaussian_kde():
     try:
@@ -39,9 +38,11 @@ def _ok_for_gaussian_kde(kind):
             from scipy.stats import gaussian_kde  # noqa
         except ImportError:
             return False
-    return True
+
+    return plotting._compat._mpl_ge_1_5_0()
 
 
+@td.skip_if_no_mpl
 class TestPlotBase(object):
 
     def setup_method(self, method):
@@ -55,6 +56,7 @@ class TestPlotBase(object):
         self.mpl_ge_1_5_0 = plotting._compat._mpl_ge_1_5_0()
         self.mpl_ge_2_0_0 = plotting._compat._mpl_ge_2_0_0()
         self.mpl_ge_2_0_1 = plotting._compat._mpl_ge_2_0_1()
+        self.mpl_ge_2_2_0 = plotting._compat._mpl_ge_2_2_0()
 
         if self.mpl_ge_1_4_0:
             self.bp_n_objects = 7
